/*------------------------------------------------------------------------------
 bin/djv_view/image_prefs_widget.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "image.h"
#include <djv_ui/button.h>
#include <djv_ui/choice.h>
#include <djv_ui/group.h>
#include <djv_ui/label.h>
#include <djv_ui/menu.h>
#include <djv_ui/style.h>
#include <djv_ui/text.h>
#include <djv_ui/vec.h>

namespace djv_view {
namespace image {

struct Prefs_Widget::_Prefs_Widget
{  
  djv_ui::choice::Widget * display;
};

Prefs_Widget::Prefs_Widget() :
  _p(new _Prefs_Widget)
{
  using namespace djv_ui;
  
  // Widgets.

  group::Box * frame_store_group =
    new group::Box(prefs_frame_store_group);
  button::Check * frame_store_file_reload =
    new button::Check(prefs_frame_store_file_reload);
  
  group::Box * transform_group =
    new group::Box(prefs_xform_group);
  button::Check * mirror_h =
    new button::Check(prefs_xform_mirror_h);
  button::Check * mirror_v =
    new button::Check(prefs_xform_mirror_v);
  choice::Widget * scale =
    new choice::Widget(label_scale(), prefs_xform_scale);
  choice::Widget * rotate =
    new choice::Widget(label_rotate(), prefs_xform_rotate);

  group::Box * color_group =
    new group::Box(prefs_color_group);
  button::Check * profile =
    new button::Check(prefs_color_profile);
  choice::Widget * channel =
    new choice::Widget(label_channel(), prefs_color_channel);
  _p->display =
    new choice::Widget(List<Str>(), prefs_color_display);

  // Layout.
  
  layout::group(List<layout::Item *>() <<
    scale <<
    rotate);
  
  layout::group(List<layout::Item *>() <<
    channel <<
    _p->display);
  
  Layout_V * layout, * layout_group, * layout_v;
  
  layout = new Layout_V(this);

  layout->add(frame_store_group);
  layout_group = new Layout_V(frame_store_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_group->add(frame_store_file_reload);

  layout->add(transform_group);
  layout_group = new Layout_V(transform_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_v = new Layout_V(layout_group);
  layout_v->margin(0);
  layout_v->add(mirror_h);
  layout_v->add(mirror_v);
  layout_group->add(scale);
  layout_group->add(rotate);
  
  layout->add(color_group);
  layout_group = new Layout_V(color_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_group->add(profile);
  layout_group->add(channel);
  layout_group->add(_p->display);

  layout->add_stretch();
 
  // Initialize.
  
  frame_store_file_reload->set(prefs()->frame_store_file_reload());
  mirror_h->set(prefs()->mirror().x);
  mirror_v->set(prefs()->mirror().y);
  scale->set(prefs()->scale());
  rotate->set(prefs()->rotate());
  profile->set(prefs()->profile());
  channel->set(prefs()->channel());
  
  widget_update();
  
  // Callbacks.

  frame_store_file_reload->sig.set(this, frame_store_file_reload_callback);
  mirror_h->sig.set(this, mirror_h_callback);
  mirror_v->sig.set(this, mirror_v_callback);
  scale->sig.set(this, scale_callback);
  rotate->sig.set(this, rotate_callback);
  profile->sig.set(this, profile_callback);
  channel->sig.set(this, channel_callback);
  _p->display->sig.set(this, display_callback);
  prefs()->display_list_sig.set(this, display_list_callback);
}

Prefs_Widget::~Prefs_Widget()
{}

void Prefs_Widget::frame_store_file_reload_callback(bool in)
{
  prefs()->frame_store_file_reload(in);
}

void Prefs_Widget::mirror_h_callback(bool in)
{
  Vec2b tmp(prefs()->mirror());
  tmp.x = in;
  prefs()->mirror(tmp);
}

void Prefs_Widget::mirror_v_callback(bool in)
{
  Vec2b tmp(prefs()->mirror());
  tmp.y = in;
  prefs()->mirror(tmp);
}

void Prefs_Widget::scale_callback(int in)
  { prefs()->scale(image::SCALE(in)); }

void Prefs_Widget::rotate_callback(int in)
  { prefs()->rotate(image::ROTATE(in)); }

void Prefs_Widget::profile_callback(bool in)
  { prefs()->profile(in); }

void Prefs_Widget::channel_callback(int in)
  { prefs()->channel(image::CHANNEL(in)); }

void Prefs_Widget::display_callback(int in)
  { prefs()->display(in - 1); }

void Prefs_Widget::display_list_callback(bool)
  { widget_update(); }

void Prefs_Widget::widget_update()
{
  callbacks(false);
  
  _p->display->list(List<Str>() <<
    prefs_color_display_default <<
    prefs()->display_name_list());
  _p->display->set(prefs()->display() + 1);

  callbacks(true);
}

}
}
