/*------------------------------------------------------------------------------
 bin/djv_view/view_prefs_widget.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "view.h"
#include <djv_ui/button.h>
#include <djv_ui/choice.h>
#include <djv_ui/color.h>
#include <djv_ui/group.h>
#include <djv_ui/label.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/style.h>
#include <djv_ui/text.h>

namespace djv_view {
namespace view {

struct Prefs_Widget::_Prefs_Widget
{
  djv_ui::color::Swatch_Widget * background_swatch;
  djv_ui::color::Swatch_Widget * grid_swatch;
  djv_ui::button::Check * hud_option [_HUD_OPTION_SIZE];
  djv_ui::color::Swatch_Widget * hud_color_foreground_swatch;
  djv_ui::color::Swatch_Widget * hud_color_background_swatch;
};

Prefs_Widget::Prefs_Widget() :
  _p(new _Prefs_Widget)
{
  using namespace djv_ui;
  
  // Widgets.

  group::Box * background_group =
    new group::Box(prefs_background_group);
  label::Widget * background_label =
    new label::Widget(prefs_background);
  _p->background_swatch =
    new color::Swatch_Widget;
  
  group::Box * resize_group =
    new group::Box(prefs_resize_group);
  choice::Widget * resize_widget =
    new choice::Widget(label_resize(), prefs_resize);
  
  group::Box * grid_group =
    new group::Box(prefs_grid_group);
  choice::Widget * grid_widget =
    new choice::Widget(label_grid(), prefs_grid);
  _p->grid_swatch =
    new color::Swatch_Widget;
  label::Widget * grid_color_label =
    new label::Widget(prefs_grid_color);
  
  group::Box * hud_group =
    new group::Box(prefs_hud_group);
  button::Check * hud_widget =
    new button::Check(prefs_hud);
  _p->hud_option[HUD_NAME] =
    new button::Check(prefs_hud_name);
  _p->hud_option[HUD_SIZE] =
    new button::Check(prefs_hud_size);
  _p->hud_option[HUD_PROXY] =
    new button::Check(prefs_hud_proxy);
  _p->hud_option[HUD_PIXEL] =
    new button::Check(prefs_hud_pixel);
  _p->hud_option[HUD_TAG] =
    new button::Check(prefs_hud_tag);
  _p->hud_option[HUD_FRAME] =
    new button::Check(prefs_hud_frame);
  _p->hud_option[HUD_SPEED] =
    new button::Check(prefs_hud_speed);
  
  choice::Widget * hud_background_widget =
    new choice::Widget(label_hud_background(), prefs_hud_background);
  
  label::Widget * hud_color_label =
    new label::Widget(prefs_hud_color);
  _p->hud_color_foreground_swatch =
    new color::Swatch_Widget;
  label::Widget * hud_color_foreground_label =
    new label::Widget(prefs_hud_color_foreground);
  _p->hud_color_background_swatch =
    new color::Swatch_Widget;
  label::Widget * hud_color_background_label =
    new label::Widget(prefs_hud_color_background);

  // Layout.
  
  layout::group(List<layout::Item *>() <<
    _p->hud_color_foreground_swatch <<
    _p->hud_color_background_swatch);
  
  Layout_V * layout, * layout_group, * layout_v;
  Layout_H * layout_h;
  
  layout = new Layout_V(this);
  
  layout->add(background_group);
  layout_group = new Layout_V(background_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_v = new Layout_V(layout_group);
  layout_v->margin(0);
  layout_v->add(background_label);
  layout_h = new Layout_H(layout_v);
  layout_h->margin(0);
  layout_h->add(_p->background_swatch);

  layout->add(resize_group);
  layout_group = new Layout_V(resize_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_group->add(resize_widget);

  layout->add(grid_group);
  layout_group = new Layout_V(grid_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_group->add(grid_widget);
  layout_v = new Layout_V(layout_group);
  layout_v->margin(0);
  layout_v->add(grid_color_label);
  layout_h = new Layout_H(layout_v);
  layout_h->margin(0);
  layout_h->add(_p->grid_swatch);
  
  layout->add(hud_group);
  layout_group = new Layout_V(hud_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_group->add(hud_widget);
  layout_v = new Layout_V(layout_group);
  layout_v->margin(0);
  for (uint i = 0; i <= _HUD_OPTION_SIZE; ++i)
    layout_v->add(_p->hud_option[i]);
  layout_group->add(hud_background_widget);
  layout_v = new Layout_V(layout_group);
  layout_v->margin(0);
  layout_v->add(hud_color_label);
  layout_h = new Layout_H(layout_v);
  layout_h->margin(0);
  layout_h->add(_p->hud_color_foreground_swatch);
  layout_h->add(hud_color_foreground_label);
  layout_h = new Layout_H(layout_v);
  layout_h->margin(0);
  layout_h->add(_p->hud_color_background_swatch);
  layout_h->add(hud_color_background_label);
  
  layout->add_stretch();
  
  // Initialize.
  
  _p->background_swatch->set(prefs()->background());
  
  resize_widget->set(prefs()->resize());

  grid_widget->set(prefs()->grid());
  _p->grid_swatch->set(prefs()->grid_color());
  
  hud_widget->set(prefs()->hud());
  for (uint i = 0; i < _HUD_OPTION_SIZE; ++i)
    _p->hud_option[i]->set(prefs()->hud_option(HUD_OPTION(i)));
  hud_background_widget->set(prefs()->hud_background());
  _p->hud_color_foreground_swatch->set(prefs()->hud_color_foreground());
  _p->hud_color_background_swatch->set(prefs()->hud_color_background());
  
  // Callbacks.
  
  _p->background_swatch->sig.set(this, background_callback);

  resize_widget->sig.set(this, resize_callback);

  grid_widget->sig.set(this, grid_callback);
  _p->grid_swatch->sig.set(this, grid_callback);
  
  hud_widget->sig.set(this, hud_callback);
  _p->hud_option[HUD_NAME]->sig.set(this, hud_name_callback);
  _p->hud_option[HUD_SIZE]->sig.set(this, hud_size_callback);
  _p->hud_option[HUD_PROXY]->sig.set(this, hud_proxy_callback);
  _p->hud_option[HUD_PIXEL]->sig.set(this, hud_pixel_callback);
  _p->hud_option[HUD_TAG]->sig.set(this, hud_tag_callback);
  _p->hud_option[HUD_FRAME]->sig.set(this, hud_frame_callback);
  _p->hud_option[HUD_SPEED]->sig.set(this, hud_speed_callback);
  hud_background_widget->sig.set(this, hud_background_callback);
  _p->hud_color_foreground_swatch->sig.set(this, hud_color_foreground_callback);
  _p->hud_color_background_swatch->sig.set(this, hud_color_background_callback);
}

Prefs_Widget::~Prefs_Widget()
{}

void Prefs_Widget::background_callback(const djv_image::Color & in)
{
  _p->background_swatch->set(in);
  prefs()->background(in);
}

void Prefs_Widget::resize_callback(int in)
  { prefs()->resize(RESIZE(in)); }

void Prefs_Widget::grid_callback(int in)
  { prefs()->grid(GRID(in)); }

void Prefs_Widget::grid_callback(const djv_image::Color & in)
{
  _p->grid_swatch->set(in);
  prefs()->grid_color(in);
}

void Prefs_Widget::hud_callback(bool in)
  { prefs()->hud(in); }

void Prefs_Widget::hud_name_callback(bool in)
  { prefs()->hud_option(HUD_NAME, in); }
void Prefs_Widget::hud_size_callback(bool in)
  { prefs()->hud_option(HUD_SIZE, in); }
void Prefs_Widget::hud_proxy_callback(bool in)
  { prefs()->hud_option(HUD_PROXY, in); }
void Prefs_Widget::hud_pixel_callback(bool in)
  { prefs()->hud_option(HUD_PIXEL, in); }
void Prefs_Widget::hud_tag_callback(bool in)
  { prefs()->hud_option(HUD_TAG, in); }
void Prefs_Widget::hud_frame_callback(bool in)
  { prefs()->hud_option(HUD_FRAME, in); }
void Prefs_Widget::hud_speed_callback(bool in)
  { prefs()->hud_option(HUD_SPEED, in); }

void Prefs_Widget::hud_background_callback(int in)
  { prefs()->hud_background(HUD_BACKGROUND(in)); }

void Prefs_Widget::hud_color_foreground_callback(const djv_image::Color & in)
{
  _p->hud_color_foreground_swatch->set(in);
  prefs()->hud_color_foreground(in);
}

void Prefs_Widget::hud_color_background_callback(const djv_image::Color & in)
{
  _p->hud_color_background_swatch->set(in);
  prefs()->hud_color_background(in);
}

}
}
