/*------------------------------------------------------------------------------
 bin/djv_view/widget.h
 
 Widgets
 -------

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_WIDGET_H
#define DJV_WIDGET_H

#include <djv_ui/choice.h>
#include <djv_ui/menu.h>
#include <djv_ui/num.h>
#include <djv_ui/num_float.h>
#include <djv_ui/text.h>
#include <djv_ui/slider.h>
#include <djv_image/time.h>

namespace djv_view {
namespace widget {
using namespace djv_base;

//------------------------------------------------------------------------------
// Frame Widget

class Frame_Widget : public djv_ui::widget::Widget
{
public:

  Frame_Widget();
  
  // Value.
  
  void set(int);
  int get() const;

  void list(const List<int> &);
  const List<int> & list() const;

  void speed(const djv_image::time::Speed &);
  const djv_image::time::Speed & speed() const;

  djv_ui::Signal<int> sig;
  
  // Widget.
  
  void text_font(const Fl_Font &);
  void text_color(const Fl_Color &);
  const Fl_Font & text_font() const;
  const Fl_Color & text_color() const;
  
  // Layout.
  
  void dirty();
  djv_ui::layout::Item * group_align();

private:

  void value_callback();
  DJV_CALLBACK(Frame_Widget, widget_callback, const Str &);
  DJV_CALLBACK(Frame_Widget, inc_callback, bool);
  DJV_CALLBACK(Frame_Widget, dec_callback, bool);

  void widget_update();

  int _value;
  List<int> _list;
  djv_image::time::Speed _speed;
  djv_ui::text::Edit * _widget;
};

//------------------------------------------------------------------------------
// Frame Slider

class Frame_Slider : public djv_ui::slider::Int
{
public:

  Frame_Slider();
  
  // Value.

  void in_out(int, int);
  int in() const;
  int out() const;
  
  void min_action();
  void max_action();
  
  void cache(const List<int> &);

  // Widget.
  
  int handle(int);
  void draw();
  
private:

  int _in, _out;
  List<int> _cache;
};

//------------------------------------------------------------------------------
// Frame Display Widget

class Frame_Display_Widget : public djv_ui::widget::Widget
{
public:

  Frame_Display_Widget();
  
  // Value.

  void set(int);
  int get() const;

  void speed(const djv_image::time::Speed &);
  const djv_image::time::Speed & speed() const;
  
  // Widget.
  
  void text_font(const Fl_Font &);
  void text_color(const Fl_Color &);
  const Fl_Font & text_font() const;
  const Fl_Color & text_color() const;
  
  // Layout.
  
  void dirty();
  
private:

  void widget_update();
  
  int _value;
  djv_image::time::Speed _speed;
  djv_ui::text::Display * _widget;
};

//------------------------------------------------------------------------------
// Speed Widget

class Speed_Widget : public djv_ui::widget::Widget
{
public:

  Speed_Widget();
  
  // Value.
  
  void set(const djv_image::time::Speed &);
  void set_default(const djv_image::time::Speed &);
  const djv_image::time::Speed & get() const;
  const djv_image::time::Speed & get_default() const;
  djv_ui::Signal<const djv_image::time::Speed &> sig;
  
  // Layout.
  
  void dirty();

private:

  DJV_CALLBACK(Speed_Widget, menu_callback, int);
  
  void widget_update();
  void menu_update();

  djv_image::time::Speed _value, _value_default;
  djv_ui::text::Display * _widget;
  djv_ui::menu::Popup * _menu;
};

//------------------------------------------------------------------------------
// Speed Display Widget

class Speed_Display_Widget : public djv_ui::widget::Widget
{
public:

  Speed_Display_Widget();
  
  // Value.
  
  void set(double);
  void dropped_frames(bool);
  
  // Layout.
  
  void dirty();
  
private:

  void widget_update();

  double _value;
  bool _dropped_frames;
  
  djv_ui::text::Display * _widget;
};

//------------------------------------------------------------------------------
// Labels

extern const Str
  label_timecode_format,
  label_frame_format,
  label_speed_format;

}
}

#endif

