/*------------------------------------------------------------------------------
 include/djv_ui/app.h
 
 User Interface Application
 --------------------------

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_APP_H
#define DJV_UI_APP_H

#include <djv_ui/app_dialog.h>
#include <djv_ui/base.h>
#include <djv_ui/dialog.h>
#include <djv_ui/icon.h>
#include <djv_ui/image.h>
#include <djv_ui/image_io.h>
#include <djv_ui/file_browser.h>
#include <djv_ui/messages_dialog.h>
#include <djv_ui/prefs.h>
#include <djv_ui/prefs_dialog.h>
#include <djv_ui/style.h>
#include <djv_ui/widget.h>
#include <djv_ui/window.h>
#include <djv_image/app.h>

namespace djv_ui {
namespace app {

//------------------------------------------------------------------------------
// Application

class App : public djv_image::app::App, public Callback_Base
{
public:

  App(const Str & name, int & argc, char ** argv) throw (Error);
  virtual ~App();
  
  // Execution.

  virtual int exec();
  virtual void exit(djv_base::app::EXIT = djv_base::app::EXIT_DEFAULT);
  void valid(bool);
  bool valid() const;

  // Information.

  virtual Str info() const;
  
  // Messages.

  virtual void message(const Str &) const;
  virtual void error(const Error &, const Str & = Str()) const;

  // XXX

  void widget_add(widget::Base *);
  void widget_del(widget::Base *);
  void window_add(window::Widget *);
  void window_del(window::Widget *);
  void window_update();
  void window_redraw();
  
  // Global.
  
  prefs::Prefs * prefs() const;
  prefs::Prefs * prefs_global() const;
  base::Global * base() const;
  base::Prefs * base_prefs() const;
  style::Style * style() const;
  style::Prefs * style_prefs() const;
  icon::Factory * icon() const;
  image::Prefs * image_prefs() const;
  image_io::Global * image_io() const;
  dialog::Global * dialog() const;
  prefs_dialog::Dialog * prefs_dialog() const;
  messages_dialog::Dialog * messages_dialog() const;
  file_browser::Dialog * file_browser() const;
  file_browser::Prefs * file_browser_prefs() const;
  app_dialog::Global * app_dialog() const;

private:

  bool _valid;

  List<widget::Base *> _widget_list;
  List<window::Widget *> _window_list;
  
  std::auto_ptr<prefs::Prefs> _prefs, _prefs_global;
  std::auto_ptr<base::Global> _base;
  std::auto_ptr<base::Prefs> _base_prefs;
  std::auto_ptr<style::Style> _style;
  std::auto_ptr<style::Prefs> _style_prefs;
  std::auto_ptr<icon::Factory> _icon;
  std::auto_ptr<image::Prefs> _image_prefs;
  std::auto_ptr<image_io::Global> _image_io;
  std::auto_ptr<dialog::Global> _dialog;
  widget::Auto_Ptr<prefs_dialog::Dialog> _prefs_dialog;
  widget::Auto_Ptr<messages_dialog::Dialog> _messages_dialog;
  widget::Auto_Ptr<file_browser::Dialog> _file_browser;
  std::auto_ptr<file_browser::Prefs> _file_browser_prefs;
  std::auto_ptr<app_dialog::Global> _app_dialog;
};

//------------------------------------------------------------------------------
// Labels

extern const Str
  label_info;

//------------------------------------------------------------------------------
// Global

App * global();

}
}

#endif
