/*------------------------------------------------------------------------------
 include/djv_ui/choice.h
 
 Choice Widgets
 --------------

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_CHOICE_H
#define DJV_UI_CHOICE_H

#include <djv_ui/widget.h>
#include <djv_ui/group.h>

namespace djv_ui {
namespace choice {

//------------------------------------------------------------------------------
// Choice Widget

class Widget : public widget::Widget
{
public:

  Widget(const List<Str> & list = List<Str>(), const Str & label = Str());
  virtual ~Widget();

  // Value.
  
  void set(int);
  int get() const;
  Signal<int> sig;

  // List.
  
  void list(const List<Str> &);
  const List<Str> & list() const;
  
  // Label.
  
  void label(widget::Widget *);
  widget::Widget * label() const;
  virtual Item * label_align();
  
  // Widget.

  virtual void text_font(const Fl_Font &);
  virtual void text_color(const Fl_Color &);
  virtual const Fl_Font & text_font() const;
  virtual const Fl_Color & text_color() const;
  
  // Debugging.
  
  virtual Str dbg_name() const { return "djv::ui::choice::Widget"; }
  
private:

  DJV_CALLBACK(Widget, widget_callback, int);
  
  List<Str> _value;
  
  struct _Widget;
  std::auto_ptr<_Widget> _p;
};

//------------------------------------------------------------------------------
// Edit Choice Widget

class Edit : public group::Frame
{
public:

  Edit(const List<Str> & list = List<Str>());
  virtual ~Edit();
  
  // Value.
  
  void set(const Str &);
  Str get() const;
  Signal<const Str &> sig;

  // List.
  
  void list(const List<Str> &);
  const List<Str> & list() const;

  // Layout.

  virtual void size_str(const Str &);
  
  // Debugging.
  
  virtual Str dbg_name() const { return "djv::ui::choice::Edit"; }
  
private:

  DJV_CALLBACK(Edit, widget_callback, const Str &);
  DJV_CALLBACK(Edit, menu_callback, int);

  void list_update();

  Str _value;
  List<Str> _list;
  
  struct _Edit;
  std::auto_ptr<_Edit> _p;
};

}
}

#endif
