/*------------------------------------------------------------------------------
 include/djv_ui/image_io.h
 
 Image I/O Widgets
 -----------------
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_IMAGE_IO_H
#define DJV_UI_IMAGE_IO_H

#include <djv_base/plugin.h>
#include <djv_image/io.h>
#include <djv_ui/widget.h>

namespace djv_ui {
namespace image_io {

//------------------------------------------------------------------------------
// Widget Base

class Base : public widget::Widget, public plugin::Plugin
{
public:

  Base();
  virtual ~Base();

  // Value.
  
  virtual void set(djv_image::io::Plugin *) = 0;
  Signal<djv_image::io::Plugin *> sig;
};

//------------------------------------------------------------------------------
// Load Widget

class Load : public Base {};

//------------------------------------------------------------------------------
// Save Widget

class Save : public Base {};

//------------------------------------------------------------------------------
// Load Factory

class Load_Factory : public Callback_Base, public plugin::Factory
{
public:

  Load_Factory();
  ~Load_Factory();
  
  Load * get(djv_image::io::Load *) const;
  List<Load *> list() const;
  
private:

  Str plugin_entry() const { return "djv_ui_image_load"; }
};

//------------------------------------------------------------------------------
// Save Factory

class Save_Factory : public Callback_Base, public plugin::Factory
{
public:

  Save_Factory();
  ~Save_Factory();
  
  Save * get(djv_image::io::Save *) const;
  List<Save *> list() const;
  
private:

  Str plugin_entry() const { return "djv_ui_image_save"; }
};

//------------------------------------------------------------------------------
// Preferences Widgets

class Prefs_Load : public widget::Widget
{
public:

  Prefs_Load();

  Signal<djv_image::io::Plugin *> sig;
  
private:

  DJV_CALLBACK(Prefs_Load, widget_callback, djv_image::io::Plugin *);
};

class Prefs_Save : public widget::Widget
{
public:

  Prefs_Save();

  Signal<djv_image::io::Plugin *> sig;
  
private:

  DJV_CALLBACK(Prefs_Save, widget_callback, djv_image::io::Plugin *);
};

//------------------------------------------------------------------------------
// Labels

extern const Str
  prefs_load_title,
  prefs_save_title;

//------------------------------------------------------------------------------
// Global

class Global : public Callback_Base
{
public:

  Global();
  ~Global();
  
  Load_Factory * load() const;
  Save_Factory * save() const;
  Signal<djv_image::io::Plugin *> sig;

private:

  std::auto_ptr<Load_Factory> _load;
  std::auto_ptr<Save_Factory> _save;
};

Global * global();

}
}

#endif

