/*------------------------------------------------------------------------------
 include/djv_ui/image_io_dpx.h
 
 DPX Image I/O Widgets
 ---------------------
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_IMAGE_IO_DPX_H
#define DJV_UI_IMAGE_IO_DPX_H

#include <djv_image/io_dpx.h>
#include <djv_ui/image_io.h>

namespace djv_ui {
namespace image_io_dpx {

//------------------------------------------------------------------------------
// Load Widget

class Load : public image_io::Load
{
public:

  Load();
  ~Load();
  
  plugin::Plugin * copy() const;
  const Str & name() const;
  
  void set(djv_image::io::Plugin *);
  
private:

  DJV_CALLBACK(Load, color_callback, int);
  DJV_CALLBACK(Load, black_callback, int);
  DJV_CALLBACK(Load, white_callback, int);
  DJV_CALLBACK(Load, gamma_callback, double);
  DJV_CALLBACK(Load, soft_clip_callback, int);
  DJV_CALLBACK(Load, callback, bool);
  
  void plugin_update();
  void widget_update();
  
  djv_image::io::Plugin * _plugin;
  djv_image::io_dpx::Load_Option _option;

  struct _Load;
  std::auto_ptr<_Load> _p;
};

//------------------------------------------------------------------------------
// Save Widget

class Save : public image_io::Save
{
public:

  Save();
  ~Save();

  plugin::Plugin * copy() const;
  const Str & name() const;

  void set(djv_image::io::Plugin *);
  
private:

  DJV_CALLBACK(Save, color_callback, int);
  DJV_CALLBACK(Save, black_callback, int);
  DJV_CALLBACK(Save, white_callback, int);
  DJV_CALLBACK(Save, gamma_callback, double);
  DJV_CALLBACK(Save, type_callback, int);
  DJV_CALLBACK(Save, endian_callback, int);
  DJV_CALLBACK(Save, callback, bool);
  
  void plugin_update();
  void widget_update();

  djv_image::io::Plugin * _plugin;
  djv_image::io_dpx::Save_Option _option;
  
  struct _Save;
  std::auto_ptr<_Save> _p;
};

//------------------------------------------------------------------------------
// Labels

extern const Str
  label_file_group,
  label_file_type,
  label_file_endian;

}
}

#endif

