/*------------------------------------------------------------------------------
 include/djv_ui/image_view.h
 
 Image Widget
 ------------
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_IMAGE_VIEW_H
#define DJV_UI_IMAGE_VIEW_H

#include <djv_image/color.h>
#include <djv_image/profile.h>
#include <djv_image/gl_data.h>
#include <djv_ui/gl.h>

namespace djv_ui {
namespace image_view {

//------------------------------------------------------------------------------
// Widget

class Widget : public gl::Widget
{
public:

  Widget();
  virtual ~Widget();
  
  // Data.

  virtual void set(const djv_image::data::Data *);
  const djv_image::data::Data * get() const;
  
  // Options.
  
  void option(const djv_image::gl_data::Option &);
  const djv_image::gl_data::Option & option() const;

  // View.
  
  virtual void view(const Vec2i &);
  const Vec2i & view() const;
  
  virtual void zoom(double);
  virtual void zoom(double, const Vec2i & focus);
  double zoom() const;

  virtual void view_zero();
  virtual void view_center();
  virtual void view_fit();

  Signal<bool> sig;

  // Widget.
  
  virtual void draw();
  
  // Functions.
  
  Box2f bbox() const;

private:

  const djv_image::data::Data * _data;
  djv_image::gl_data::Option _option;
  Vec2i _view;
  double _zoom;
  djv_image::gl_data::Data _gl;
};

}
}

#endif

