/*------------------------------------------------------------------------------
 include/djv_ui/label.h
 
 Label Widgets
 -------------

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_LABEL_H
#define DJV_UI_LABEL_H

#include <djv_ui/widget.h>

namespace djv_ui {
namespace label {

//------------------------------------------------------------------------------
// Label

class Widget : public widget::Widget
{
public:

  Widget(const Str & = Str());
  virtual ~Widget();
  
  // Value.

  void set(const Str &);
  const Str & get() const;

  // Widget.

  virtual void draw();
  
  // Layout.

  virtual void size_str(const Str &);
  virtual const Str & size_str() const;

  virtual void dirty();
  virtual Item * group_align();
  virtual Item * label_align();
  
  // Debugging.
  
  virtual Str dbg_name() const { return "djv::ui::label::Widget"; }
  
private:

  Str _value;
  int _value_width;
  Str _size_str;
};

//------------------------------------------------------------------------------
// Text Label

class Text : public widget::Widget
{
public:

  Text(const Str & = Str());
  virtual ~Text();
  
  // Value.

  void set(const Str &);
  const Str & get() const;

  // Widget.

  virtual void draw();
  
  // Layout.

  virtual void dirty();
  virtual void geom(const Box2i &);
  virtual const Box2i & geom() const;
  virtual const Vec2i & size_min() const;
  virtual Item * group_align();
  virtual Item * label_align();
  
  // Debugging.
  
  virtual Str dbg_name() const { return "djv::ui::label::Text"; }
  
private:

  Vec2i text_hint(const Vec2i &) const;

  Str _value;
  List<Str> _word_list;
  List<int> _word_size_list;
  int _word_size_max, _space_size, _text_height;
  Vec2i _size_min, _text_hint;
};

//------------------------------------------------------------------------------
// Image Label

class Image : public widget::Widget
{
public:

  Image(Fl_Image * = 0);
  virtual ~Image();
  
  // Value.

  void set(Fl_Image *);
  Fl_Image * get() const;
  
  // Widget.

  virtual void draw();
  
  // Layout.

  virtual void dirty();
  virtual Item * group_align();
  virtual Item * label_align();
  
private:

  Fl_Image * _value;
};

}
}

#endif
