/*------------------------------------------------------------------------------
 include/djv_ui/prefs_dialog.h
 
 Preferences Dialog
 ------------------

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_PREFS_DIALOG_H
#define DJV_UI_PREFS_DIALOG_H

#include <djv_ui/dialog.h>

namespace djv_ui {
namespace prefs_dialog {

//------------------------------------------------------------------------------
// Preferences Widgets

class Widget : public widget::Widget
{
public:

  Widget();
  
private:

  DJV_CALLBACK(Widget, time_units_callback, int);
  DJV_CALLBACK(Widget, speed_callback, int);
  DJV_CALLBACK(Widget, filter_callback, int);
  DJV_CALLBACK(Widget, filter_default_min_callback, int);
  DJV_CALLBACK(Widget, filter_default_mag_callback, int);
  DJV_CALLBACK(Widget, filter_custom_min_callback, int);
  DJV_CALLBACK(Widget, filter_custom_mag_callback, int);
  DJV_CALLBACK(Widget, color_callback, int);
  DJV_CALLBACK(Widget, color_foreground_callback, const djv_image::Color &);
  DJV_CALLBACK(Widget, color_background_callback, const djv_image::Color &);
  DJV_CALLBACK(Widget, color_background2_callback, const djv_image::Color &);
  DJV_CALLBACK(Widget, color_select_callback, const djv_image::Color &);
  DJV_CALLBACK(Widget, size_callback, int);
  DJV_CALLBACK(Widget, tooltip_callback, bool);
};

//------------------------------------------------------------------------------
// Preferences Dialog

class Dialog : public dialog::Base
{
public:
  
  Dialog();
  virtual ~Dialog();

  // Tabs.
  
  void add(const Str & name, widget::Base *);
  void add_shortcut(const Str & name, widget::Base *);
  
  // Widget.
  
  virtual void show();
  
private:
  
  DJV_CALLBACK(Dialog, close_callback, bool);

  struct _Dialog;
  std::auto_ptr<_Dialog> _p;
};

//------------------------------------------------------------------------------
// Labels

extern const Str
  dialog_title;

extern const Str
  prefs_title,
  prefs_time_group,
  prefs_time_units,
  prefs_speed,
  prefs_speed_unit,
  prefs_filter_group,
  prefs_filter,
  prefs_filter_default,
  prefs_filter_custom,
  prefs_filter_unit,
  prefs_style_group,
  prefs_color,
  prefs_color_custom,
  prefs_color_foreground,
  prefs_color_background,
  prefs_color_background2,
  prefs_color_select,
  prefs_size,
  prefs_misc_group,
  prefs_tooltip;

//------------------------------------------------------------------------------
// Global

Dialog * global();

}
}

#endif

