/*------------------------------------------------------------------------------
 include/djv_ui/slider.h

 Slider Widgets
 --------------

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_SLIDER_H
#define DJV_UI_SLIDER_H

#include <djv_ui/group.h>
#include <djv_ui/num.h>

namespace djv_ui {
namespace slider {

//------------------------------------------------------------------------------
// Widget Base

class Base : public group::Frame
{
public:

  Base();
  virtual ~Base();
  
  // Widget.

  virtual int handle(int);
  
  // Layout.

  virtual void handle_size(int);
  virtual int handle_size() const;
  virtual Box2i slider_geom() const;

  virtual void dirty();
  
  // Debugging.
  
  virtual Str dbg_name() const { return "djv::ui::slider::Base"; }
  
protected:

  virtual void handle_draw(double value, const Box2i &);
  
private:
  
  int _handle_size;
};

//------------------------------------------------------------------------------
// Integer Slider

class Int : public Base, public num::Int_Base
{
public:

  Int();
  virtual ~Int();
  
  // Value.
  
  Signal<int> sig;
  
  // Widget.

  virtual int handle(int);
  virtual void draw();

private:

  void value_callback();
};

//------------------------------------------------------------------------------
// Float Slider

class Float : public Base, public num::Float_Base
{
public:
  
  Float();
  virtual ~Float();
  
  // Value.
  
  Signal<double> sig;

  // Widget.
 
  virtual int handle(int);
  virtual void draw();

private:

  void value_callback();
};

}
}

#endif

