/*------------------------------------------------------------------------------
 include/djv_ui/text.h
 
 Text Widgets
 ------------

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_TEXT_WIDGET_H
#define DJV_UI_TEXT_WIDGET_H

#include <djv_ui/group.h>

namespace djv_ui {
namespace text {

//------------------------------------------------------------------------------
// Text Edit

class Edit : public group::Frame
{
public:

  Edit(const Str & = Str());
  virtual ~Edit();
  
  // Value.
  
  void set(const Str &);
  const Str & get() const;
  Signal<const Str &> sig, action_sig, change_sig;
  
  // Widget.

  virtual void color(const Fl_Color &);
  virtual const Fl_Color & color() const;

  virtual void text_font(const Fl_Font &);
  virtual void text_color(const Fl_Color &);
  virtual const Fl_Font & text_font() const;
  virtual const Fl_Color & text_color() const;
  
  virtual int handle(int);
  
  // Layout.

  virtual void size_str(const Str &);
  virtual const Str & size_str() const;
  
  virtual void dirty();
  virtual void geom(const Box2i &);
  virtual const Box2i & geom() const;
  
  // Debugging.
  
  virtual Str dbg_name() const { return "djv::ui::text::Widget"; }

private:

  DJV_FL_WIDGET_CALLBACK(Edit, widget_callback);

  void widget_update();

  Str _value;
  Str _size_str;
  
  struct _Edit;
  std::auto_ptr<_Edit> _p;
};

//------------------------------------------------------------------------------
// Text Display

class Display : public group::Frame
{
public:

  Display(const Str & = Str());
  virtual ~Display();

  // Value.

  void set(const Str &);
  const Str & get() const;
  
  // Widget.
  
  virtual void text_font(const Fl_Font &);
  virtual void text_color(const Fl_Color &);
  virtual const Fl_Font & text_font() const;
  virtual const Fl_Color & text_color() const;
  
  // Layout.

  virtual void size_str(const Str &);
  virtual const Str & size_str() const;

  virtual void dirty();
  virtual void geom(const Box2i &);
  virtual const Box2i & geom() const;
  
  // Debugging.
  
  virtual Str dbg_name() const { return "djv::ui::text::Display"; }

private:

  class Widget;

  void widget_update();

  Str _value;
  Str _size_str;
  Widget * _widget;
};

//------------------------------------------------------------------------------
// Multiline Text Display

class Multiline_Display : public group::Frame
{
public:

  Multiline_Display(const Str & = Str());
  virtual ~Multiline_Display();
  
  // Value.

  void set(const Str &);
  const Str & get() const;
  
  // Widget.
  
  virtual void text_font(const Fl_Font &);
  virtual void text_color(const Fl_Color &);
  virtual const Fl_Font & text_font() const;
  virtual const Fl_Color & text_color() const;
  
  // Layout.

  virtual void dirty();
  virtual void geom(const Box2i &);
  virtual const Box2i & geom() const;
  
  // Debugging.
  
  virtual Str dbg_name() const { return "djv::ui::text::Multiline_Display"; }

private:

  class Widget;

  void widget_update();

  Str _value;
  Widget * _widget;
};

}
}

#endif

