/*------------------------------------------------------------------------------
 lib/djv_base/dbg.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_base/dbg.h>
#include <djv_base/str.h>
#include <stdio.h>
#include <stdlib.h>

namespace djv_base {
namespace dbg {

void _djv_assert(const char * file, int line)
{
  ::printf("ASSERT: File \"%s\", line %d.\n", file, line);
  ::fflush(stdout);

  ::abort();
}

namespace { int indent = 0; } // XXX Global variable.

void Dbg::init(const Str & in)
{
  *this << LINE_BEGIN << in << LINE_END;
  *this << LINE_BEGIN << "{" << LINE_END;

  indent += 2;
}

Dbg::Dbg(const Str & in)
{
  init(in);
}

Dbg::Dbg(const Str & prefix, const Str & in)
{
  init(prefix + ": " + in);
}

Dbg::~Dbg()
{
  _timer.check();

  indent -= 2;
  
  *this <<
    LINE_BEGIN << "} (" <<
    _timer.seconds() <<
    " seconds, " <<
    _timer.fps() <<
    " fps)" <<
    LINE_END;
}

Dbg & Dbg::operator << (LINE in)
{
  switch (in)
  {
    case LINE_BEGIN:
      _tmp = "dbg " + Str(' ', indent);
      break;

    case LINE_END:
      ::printf("%s\n", _tmp.c_str());
      ::fflush(stdout);
      break;
  }
  
  return *this;
}

void Dbg::add(const Str & in) { _tmp += in; }

Dbg & operator << (Dbg & dbg, const Str & in)
{
  dbg.add(in);
  return dbg;
}

Dbg & operator << (Dbg & dbg, const char * in)
{
  dbg.add(in);
  return dbg;
}

Dbg & operator << (Dbg & dbg, bool in)
{
  return dbg << (in ? "true" : "false");
}

Dbg & operator << (Dbg & dbg, int in)
{
  return dbg << str::int_to_str(in);
}

Dbg & operator << (Dbg & dbg, uint in)
{
  return dbg << str::uint_to_str(in);
}

Dbg & operator << (Dbg & dbg, double in)
{
  return dbg << str::float_to_str(in);
}

Dbg & operator << (Dbg & dbg, const List<Str> & in)
{
  return dbg << str::join(in, " ");
}

namespace {

Str bits(uint32_t in, int size)
{
  Str out;
  for (int i = 0; i < size; ++i)
    out += ((in >> i) & 1) ? '1' : '0';
  return out;
}

}

Str u8(uint8_t in) { return bits(in, 8); }
Str u16(uint16_t in) { return bits(in, 16); }
Str u32(uint32_t in) { return bits(in, 32); }

}
}
