/*------------------------------------------------------------------------------
 lib/djv_base/file_path.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include <djv_base/file.h>
#include <FL/filename.H>

namespace djv_base {
namespace file {

Str path_absolute(const Str & in)
{
  DJV_DBG_OBJ("path_absolute");
  DJV_DBG_PRN("in = " << in);
  
  char * p = 0;
  
  char tmp [str::cstr_size] = "";
  fl_filename_absolute(tmp, str::cstr_size, in.c_str());
  p = tmp;
  
  // XXX

#if defined(WIN32)
  TCHAR tmp2 [4096];
  GetFullPathName(p, 4096, tmp2, 0);
  p = tmp2;
#endif

  Str out(p);
  DJV_DBG_PRN("out = " << out);
  return out;
}

bool path_is_absolute(const Str & in)
{
  if (! in.size()) return false;

  // XXX Windows drive letter.
  
  if (1 == in.find(':', 0)) return true;

  return list::find(in[0], path_separator_list) != -1;
}

Str path_fix(const Str & in)
{
  DJV_DBG_OBJ("path_fix");
  DJV_DBG_PRN("in = " << in);

  // Let FLTK do it's stuff.
  
  char tmp [str::cstr_size], path [str::cstr_size];
  fl_filename_expand(tmp, str::cstr_size, in.c_str());
  fl_filename_absolute(path, str::cstr_size, tmp);

  // XXX Keep original path separators?

  //for (char * p = path; *p && (p - path) < str::cstr_size; ++p)
  //  if (path_separator_list[0] == *p)
  //    *p = path_separator[0];
  
  DJV_DBG_PRN("path = " << path);
  
  // Split path into pieces.
  
  List<Str> list = str::split(path, path_separator_list);
  
  // XXX Keep extra root piece for Windows.

  Str root;
  if (
    (path_separator_list[0] == path[0] && path_separator_list[0] == path[1]) ||
    (path_separator_list[1] == path[0] && path_separator_list[1] == path[1])
  ) {
    root = path_separator;
  }
  else if (list.size())
  {
    if (list[0].find(':', 0) != Str::npos)
    {
      root = list[0];
      list.erase(list.begin());
    }
  }

  // Remove extra pieces.
  
  List<Str> list_tmp;
  for (List<Str>::const_iterator i = list.begin(); i < list.end(); ++i)
    if (".." == *i && list_tmp.size())
    {
      if (list_tmp[list_tmp.size() - 1] != "..")
        list_tmp.erase(list_tmp.end() - 1);
      else
        list_tmp += *i;
    }
    else if ("." == *i && list_tmp.size()) ;
    else list_tmp += *i;
  
  // Combine pieces.

  Str out = root + path_separator + str::join(list_tmp, path_separator);
  if (list_tmp.size()) out += path_separator;
  DJV_DBG_PRN("out = " << out);
  return out;
}

/*Str path_posix(const Str & in)
{
  Str out = in;
  for (uint i = 0; i < out.size(); ++i)
    if (path_separator_list[1] == out[i])
      out[i] = path_separator_list[0];
  return out;
}*/

}
}

