/*------------------------------------------------------------------------------
 lib/djv_base/file_split.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include <djv_base/file.h>

namespace djv_base {
namespace file {

// XXX

inline bool _path_separator(char in)
{
  return in == path_separator_list[0] || in == path_separator_list[1];
}

inline bool _seq_valid(char in)
{
  return str::is_digit(in) || '#' == in;
}

inline bool _seq_separator(char in)
{
  return '-' == in || ',' == in;
}

inline bool _pad_match(const Str & a, const Str & b)
{
  DJV_DBG_OBJ("_pad_match");
  DJV_DBG_PRN("a = " << a);
  DJV_DBG_PRN("b = " << b);
  
  return ((a.size() > 1 && '0' == a[0]) || (b.size() > 1 && '0' == b[0])) ?
    (a.size() == b.size()) : true;
}

void split(const Str & in, Str * path, Str * base, Str * number,
  Str * extension)
{
  DJV_DBG_OBJ("split");
  DJV_DBG_PRN("in = " << in);
  DJV_DBG_PRN("size = " << in.size());
  
  path->resize(0);
  base->resize(0);
  number->resize(0);
  extension->resize(0);

  const int size = int(in.size());
  if (! size) return;

  // Extension.

  int i = size - 1;
  int tmp = i;
  for (; in[i] != '.' && ! _path_separator(in[i]) && i > 0; --i) ;
  if (i > 0 && '.' == in[i])
  {
    *extension = Str(in, i, tmp - i + 1);
    --i;
  } else
    i = size - 1;
  DJV_DBG_PRN("extension = " << *extension);

  // Number.

  if (i >= 0 && _seq_valid(in[i]))
  {
    tmp = i;
    int separator = -1;
    Str word;
    for (; i > 0; --i)
    {
      if (! _seq_valid(in[i - 1]) || _seq_separator(in[i - 1]))
      {
        if (separator != -1 && ! _pad_match(Str(in, i, separator - i), word))
        {
          i = separator + 1;
          break;
        }
        else
        {
          word = Str(in, i, -1 == separator ? (tmp - i + 1) : (separator - i));
          separator = i - 1;
        }
      }
      if (! (_seq_valid(in[i - 1]) || _seq_separator(in[i - 1])))
        break;
    }
    *number = Str(in, i, tmp - i + 1);
    --i;
  }
  DJV_DBG_PRN("number = " << *number);

  // Base.
  
  if (i >= 0 && ! _path_separator(in[i]))
  {
    tmp = i;
    for (; i > 0 && ! _path_separator(in[i - 1]); --i) ;
    *base = Str(in, i, tmp - i + 1);
    --i;
  }
  DJV_DBG_PRN("base = " << *base);

  // Path.

  if (i >= 0)
    *path = Str(in, 0, i + 1);
  DJV_DBG_PRN("path = " << *path);
}

}
}
