/*------------------------------------------------------------------------------
 lib/djv_base/sys.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include <djv_base/app.h>
#include <djv_base/file.h>
#include <djv_base/sys.h>
#if defined(WIN32)
#include <windows.h>
#else
#include <sys/ioctl.h>
#include <sys/termios.h>
#include <unistd.h>
#endif
#include <stdlib.h>

namespace djv_base {
namespace sys {

Str env(const Str & in)
{
  const char * p = ::getenv(in.c_str());
  return Str(p ? p : "");
}

List<Str> search_path()
{
  DJV_DBG_OBJ("search_path");
  
  List<Str> out;

  // Environment variables.

  Str path = sys::env("DJV_PATH");
  if (! path.empty())
  {
    DJV_DBG_PRN("DJV_PATH = " << path);
    out += str::split(path, file::list_separator_list);
  }

  path = sys::env("LD_LIBRARY_PATH");
  if (! path.empty())
  {
    DJV_DBG_PRN("LD_LIBRARY_PATH = " << path);
    out += str::split(path, file::list_separator_list);
  }

  // Installation path.
  
#if defined(DJV_SEARCH_PATH)

  DJV_DBG_PRN("DJV_SEARCH_PATH = " << DJV_SEARCH_PATH);

  List<Str> tmp = str::split(DJV_SEARCH_PATH, file::list_separator_list);
  for (uint i = 0; i < tmp.size(); ++i)
    if (! tmp[i].empty())
      out += tmp[i];

#endif

  // Application path.
  
  out += app::global()->path();
  
  // XXX Mac OS X plugin path.

#if defined(DJV_MACOSX)
  out += app::global()->path() + "../PlugIns";
#endif

  // Current directory.

  out += ".";

  DJV_DBG_PRN("out = " << out);
  return out;
}

int terminal_width()
{
  int out = 80;

#if defined(WIN32)

  HANDLE h = ::GetStdHandle(STD_OUTPUT_HANDLE);
  if (h != INVALID_HANDLE_VALUE)
  {
    CONSOLE_SCREEN_BUFFER_INFO info;
    if (::GetConsoleScreenBufferInfo(h, &info))
      out = info.dwSize.X;
  }

#else

  struct winsize ws;
  ws.ws_col = 0;
  if (ioctl(STDOUT_FILENO, TIOCGWINSZ, &ws) == -1)
    ws.ws_col = 80;

  out = ws.ws_col;

#endif

  return out;
}

}
}
