/*------------------------------------------------------------------------------
 lib/djv_image/data.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include <djv_image/color.h>
#include <djv_image/data.h>

namespace djv_image {
namespace data {

void Info::init()
{
  proxy = PROXY_NONE;
  pixel = PIXEL(0);
  bgr = false;
  align = 1;
  endian = mem::endian();
}

Info::Info()
{
  init();
}

Info::Info(const Vec2i & size, PIXEL pixel)
{
  init();
  
  this->size = size;
  this->pixel = pixel;
}

Info::Info(const Str & name, const Vec2i & size, PIXEL pixel)
{
  init();
  
  this->name = name;
  this->size = size;
  this->pixel = pixel;
}

void Data::init()
{
  _channels = 0;
  _bytes_pixel = 0;
  _bytes_scanline = 0;
  _bytes_data = 0;
  _p = 0;
}

Data::Data()
{
  DJV_DBG_OBJ("Data::Data");
  
  init();
}

Data::Data(const Data & in)
{
  DJV_DBG_OBJ("Data::Data");

  init();
  
  set(in);
}

Data::Data(const Info & in, const uint8_t * data)
{
  DJV_DBG_OBJ("Data::Data");

  init();
  
  set(in, data);
}

Data::~Data() {}

Data & Data::operator = (const Data & in)
{
  if (&in != this) set(in);
  return *this;
}

void Data::set(const Data & in)
{
  set(in._info);
  mem::copy(in.data(), data(), _bytes_data);
}

void Data::set(const Info & in, const uint8_t * data)
{
  DJV_DBG_OBJ("Data::Data");
  DJV_DBG_PRN("in = " << in);

  _info = in;
  
  _channels = pixel::channels(_info.pixel);
  _bytes_pixel = pixel::bytes(_info.pixel);
  _bytes_scanline = data::bytes_scanline(_info);
  _bytes_data = data::bytes_data(_info);
  DJV_DBG_PRN("channels = " << _channels);
  DJV_DBG_PRN("bytes pixel = " << _bytes_pixel);
  DJV_DBG_PRN("bytes scanline = " << _bytes_scanline);
  DJV_DBG_PRN("bytes data = " << _bytes_data);

  if (data)
  {
    _data.size(0);
    _p = data;
  }
  else
  {
    _data.size(_bytes_data);
    _p = _data();
  }
}

void Data::zero()
{ 
  DJV_DBG_OBJ("Data::zero");

  _data.zero();
}

}
}
