/*------------------------------------------------------------------------------
 lib/djv_image/gl_data_program.h

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef _DJV_IMAGE_GL_DATA_PROGRAM_H
#define _DJV_IMAGE_GL_DATA_PROGRAM_H

#include <djv_image/gl_data.h>

namespace djv_image {
namespace gl_data {
namespace program {

//------------------------------------------------------------------------------
// LUT

class Lut
{
public:

  Lut();
  ~Lut();
  
  void init(const djv_image::Info &) throw (Error);
  void init(const djv_image::Data &) throw (Error);
  void copy(const djv_image::Data &);
  void bind();
  const djv_image::Info & info() const;
  GLuint id() const;

private:

  void del();
  
  djv_image::Info _info;
  int _size;
  GLuint _id;
};

//------------------------------------------------------------------------------
// Texture

class Texture
{
public:

  Texture();
  ~Texture();
  
  void init(const djv_image::Info &, GLenum = GL_LINEAR, GLenum = GL_LINEAR)
    throw (Error);
  void init(const djv_image::Data &, GLenum = GL_LINEAR, GLenum = GL_LINEAR)
    throw (Error);
  void copy(const djv_image::Data &);
  void copy(const Vec2i &);
  void bind();
  const djv_image::Info & info() const;
  GLenum min() const;
  GLenum mag() const;
  GLuint id() const;

private:

  void del();
  
  djv_image::Info _info;
  GLenum _min, _mag;
  GLuint _id;
};

//------------------------------------------------------------------------------
// Shader

class Shader
{
public:

  Shader();
  ~Shader();
  
  void init(const Str &) throw (Error);
  void bind();
  static void unbind();
  GLhandleARB program() const;

private:

  void del();

  GLhandleARB _id, _program;
};

//------------------------------------------------------------------------------
// Program

class Program
{
public:

  void draw(const djv_image::Data &, const Option &) throw (Error);

private:

  class State_Default
  {
  public:
  
    State_Default();
    State_Default(const Option &);
    
    bool operator != (const State_Default &) const;
    
  private:
  
    profile::PROFILE _profile;
    CHANNEL _channel;
    Display _display;
    Filter::Default _filter;
    bool _clamp;
    bool _init;

  } _state_default;

  class State_Custom
  {
  public:
  
    State_Custom();
    State_Custom(const Vec2i &, int proxy_scale, const Option &);
    
    bool operator != (const State_Custom &) const;
    
  private:
  
    Vec2i _size;
    int _proxy_scale;
    profile::PROFILE _profile;
    CHANNEL _channel;
    Display _display;
    Filter::Custom _filter;
    bool _clamp;
    bool _init;

  } _state_custom;
  
  Lut _lut_profile, _lut_display;
  Texture _scale_x_contrib, _scale_y_contrib;
  Shader _scale_x_shader, _scale_y_shader;
  Texture _texture;
  Shader _shader;
  std::auto_ptr<gl_offscreen::Buffer> _buffer;
};

//------------------------------------------------------------------------------
// Functions

void scale_contrib(
  int input,
  int output,
  const Filter::Custom & filter,
  djv_image::Data *
);

void quad(
  const Vec2i & size,
  const Vec2b & mirror = Vec2b(),
  int proxy_scale = 1
);

}
}
}

#endif
