/*------------------------------------------------------------------------------
 lib/djv_image/io_cineon_info_etc.cxx
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "io_cineon_info.h"

namespace djv_image {
namespace io_cineon {
namespace info {

void zero(int32_t * in)
{
  *((uint32_t *)in) = 0x80000000;
}

void zero(float * in)
{
  *((uint32_t *)in) = 0x7F800000;
}

void zero(char * in, int size)
{
  mem::zero(in, size);
}

bool valid(const uint8_t * in)
{
  return *in != 0xff;
}

bool valid(const uint16_t * in)
{
  return *in != 0xffff;
}

namespace {

// XXX Catch uninitialized values.

const int32_t _int_max = 1000000;
const float _float_max = 1000000.0;
const char _char_min = 32;
const char _char_max = 126;

}

bool valid(const uint32_t * in)
{
  return
    *in != 0xffffffff &&
    *in < uint32_t(_int_max);
}

bool valid(const int32_t * in)
{
  return
    *in != int32_t(0x80000000) &&
    *in > -_int_max &&
    *in < _int_max;
}

bool valid(const float * in)
{
  return
    *((uint32_t *)in) != 0x7F800000 &&
    *in > -_float_max &&
    *in < _float_max;
}

bool valid(const char * in, int size)
{
  const char * p = in;
  const char * const end = p + size;
  for (; *p && p < end; ++p)
    if (*p < _char_min || *p > _char_max)
      return false;
  return size ? (in[0] != 0) : false;
}

Str to_str(const char * in, int size)
{
  const char * p = in;
  const char * const end = p + size;
  for (; *p && p < end; ++p) ;
  return Str(in, p - in);
}

Str dbg(uint8_t in)
{
  return valid(&in) ? str::int_to_str(in) : "-";
}

Str dbg(uint16_t in)
{
  return valid(&in) ? str::int_to_str(in) : "-";
}

Str dbg(uint32_t in)
{
  return valid(&in) ? str::int_to_str(in) : "-";
}

Str dbg(int32_t in)
{
  return valid(&in) ? str::int_to_str(in) : "-";
}

Str dbg(float in)
{
  return valid(&in) ? str::float_to_str(in) : "-";
}

Str dbg(const char * in, int size)
{
  return valid(in, size) ? Str(in, size) : "-";
}

}
}
}
