/*------------------------------------------------------------------------------
 lib/djv_image/io_tga_load.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include <djv_image/io_tga.h>

namespace djv_image {
namespace io_tga {

Load::Load()
{
  _io.endian(mem::endian() != mem::LSB);
}

io::Plugin * Load::copy() const { return new Load; }

const Str & Load::name() const { return io_tga::name; }

const List<Str> & Load::extension_list() const
  { return io_tga::extension_list; }

void Load::open(const File & in, Info * info, Tag *, Time * time) throw (Error)
{
  DJV_DBG_OBJ("Load::open");
  DJV_DBG_PRN("in = " << in);

  _file = in;
  _open(_file.get(_file.seq.start()), info);

  if (file::SEQ == _file.type)
    time->list = _file.seq.list;
}

void Load::open(const File & in, List<Info> * info, Tag * tag, Time * time)
  throw (Error)
{
  io::Load::open(in, info, tag, time);
}

void Load::_open(const File & in, Info * info) throw (Error)
{
  DJV_DBG_OBJ("Load::_open");
  DJV_DBG_PRN("in = " << in);
  
  _io.open(in, djv_base::io::READ);
  
  info->name = in;
  info_load(&_io, info, &_compress);
}

const Image * Load::get(int frame, int, data::PROXY proxy) throw (Error)
{
  DJV_DBG_OBJ("Load::get");
  DJV_DBG_PRN("frame = " << frame);
  DJV_DBG_PRN("proxy = " << proxy);
  
  // Open.
  
  Info info;
  _open(_file.get(frame != -1 ? frame : _file.seq.start()), &info);

  // Read.
  
  _io.readahead();
  
  Data * data = proxy ? &_tmp : &_image;
  
  if (! _compress)
  {
    const uint8_t * p = _io.mmap_p();
    _io.seek(data::bytes_data(info));
    data->set(info, p);
    
    //data->set(info);
    //_io.get(data->data(), data->bytes_data());
  }
  else
  {
    data->set(info);

    const uint8_t * p = _io.mmap_p(), * const end = _io.mmap_end();
    const int channels = pixel::channels(info.pixel);
    for (int y = 0; y < info.size.y; ++y)
    {
      DJV_DBG_PRN("y = " << y);
      p = rle_load(p, end, data->data(0, y), info.size.x, channels);
      if (! p) throw Error(name(), io::error_read);
    }
  }

  if (proxy)
  {
    info.size = data::proxy_scale(info.size, proxy);
    info.proxy = proxy;
    _image.set(info);
    data::proxy_scale(_tmp, &_image, proxy);
  }
  
  DJV_DBG_PRN("image = " << _image);
  return &_image;
}

}
}
