/*------------------------------------------------------------------------------
 lib/djv_ui/base.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include <djv_ui/base.h>
#include <djv_ui/prefs.h>
#include <FL/Fl_Tooltip.H>

namespace djv_ui {
namespace base {

Cursor::Cursor(const Fl_Cursor & in)
{
  cursor(in);
}

Cursor::~Cursor()
{
  cursor(FL_CURSOR_DEFAULT);
}

Tooltip::Tooltip() :
  _enable(prefs()->tooltip())
{
  enable_update();
}

void Tooltip::enable(bool in)
{
  if (in == _enable) return;
  
  _enable = in;
  
  enable_update();
}

bool Tooltip::enable() const { return _enable; }

void Tooltip::enable_update()
{
  if (_enable) Fl_Tooltip::enable();
  else Fl_Tooltip::disable();
}

Prefs::Prefs() :
  _tooltip(true)
{
  prefs::Prefs prefs(prefs::prefs_global(), "base");
  prefs::get(&prefs, "tooltip", &_tooltip);
}

Prefs::~Prefs()
{
  prefs::Prefs prefs(prefs::prefs_global(), "base");
  prefs::set(&prefs, "tooltip", _tooltip);
}

void Prefs::tooltip(bool in)
{
  if (in == _tooltip) return;
  
  _tooltip = in;
  
  base::global()->tooltip()->enable(_tooltip);
}
  
bool Prefs::tooltip() const { return _tooltip; }

Global::Global() :
  _tooltip(new Tooltip)
{}

Global::~Global()
{}
  
Tooltip * Global::tooltip() const { return _tooltip.get(); }

}
}
