/*------------------------------------------------------------------------------
 lib/djv_ui/file_browser_callback.cxx
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "file_browser.h"
#include <djv_base/dir.h>

namespace djv_ui {
namespace file_browser {

void Dialog::widget_callback(const Str & in)
{
  _value = in;
}

void Dialog::widget_action_callback(const Str & in)
{
  File tmp(djv_base::file::path_is_absolute(in) ? in : Str(_dir + in));
  if (djv_base::file::DIR == tmp.type)
  {
    dir(tmp);
  }
  else
  {
    _value = in;
    ok_callback(true);
  }
}

void Dialog::dir_callback(const Str & in) { dir(in); }

void Dialog::browser_callback(const File & in)
{
  _value = in;

  widget_update();
}

void Dialog::browser_action_callback(const File & in)
{
  if (djv_base::file::DIR == in.type)
  {
    dir(in);
  }
  else
  {
    _value = in;

    widget_update();
    ok_callback(true);
  }
}

void Dialog::columns_callback(const List<int> & in)
{
  _p->browser->column_list(in);
}

void Dialog::columns_action_callback(int in)
{
  if (in == _sort)
    sort_reverse(! _sort_reverse);
  else
    sort(SORT(in));
}

void Dialog::up_callback(bool)
{
  Dir tmp(_dir);
  tmp.up();
  dir(tmp.get());
}

void Dialog::_up_callback() { up_callback(true); }

void Dialog::prev_callback(bool) { dir(_dir_prev); }
void Dialog::_prev_callback() { prev_callback(true); }

void Dialog::_recent_callback()
{
  const List<Str> & recent = prefs()->recent();
  const int i = list::find(_p->menu->item(), _p->menu_recent);
  if (i >= 0 && i < int(recent.size())) dir(recent[i]);
}

void Dialog::current_callback(bool) { dir(Dir(".").get()); }
void Dialog::_current_callback() { current_callback(true); }

#if ! defined(WIN32)

void Dialog::home_callback(bool) { dir(djv_base::dir::home()); }
void Dialog::_home_callback() { home_callback(true); }

#endif

void Dialog::desktop_callback(bool) { dir(djv_base::dir::desktop()); }
void Dialog::_desktop_callback() { desktop_callback(true); }

#if defined(WIN32)

void Dialog::drive_callback(bool)
{
  const List<Str> drive_list = djv_base::dir::drive_list();
  const int i = list::find(_p->menu->item(), _p->menu_drive);
  if (i >= 0 && i < int(drive_list.size())) dir(drive_list[i]);
}

void Dialog::_drive_callback() { drive_callback(true); }

#else

void Dialog::root_callback(bool) { dir(djv_base::dir::root()); }
void Dialog::_root_callback() { root_callback(true); }

#endif

void Dialog::reload_callback(bool) { dir_update(); }
void Dialog::_reload_callback() { reload_callback(true); }

void Dialog::_image_callback()
{
  image(IMAGE(list::find(_p->menu->item(), _p->menu_image)));
}

void Dialog::_seq_callback()
{
  seq(djv_base::file::SEQ_COMPRESS(list::find(_p->menu->item(), _p->menu_seq)));
}

void Dialog::_hidden_callback() { hidden(! _hidden); }

void Dialog::_sort_callback()
{
  sort(SORT(list::find(_p->menu->item(), _p->menu_sort)));
}

void Dialog::_sort_reverse_callback() { sort_reverse(! _sort_reverse); }
void Dialog::_sort_dir_callback() { sort_dir(! _sort_dir); }

void Dialog::_bookmark_callback()
{
  const List<Str> & bookmark = prefs()->bookmark();
  const int i = list::find(_p->menu->item(), _p->menu_bookmark);
  if (i >= 0 && i < int(bookmark.size())) dir(bookmark[i]);
}

void Dialog::_bookmark_add_callback()
{
  prefs()->bookmark_add(_dir);

  menu_update();
}

void Dialog::bookmark_del_callback(int in)
{
  List<Str> bookmark = prefs()->bookmark();
  if (in >= int(bookmark.size())) return;

  List<Str>::iterator i = bookmark.begin();
  for (int j = 0; j < in; ++j, ++i) ;
  bookmark.erase(i);
  prefs()->bookmark(bookmark);

  menu_update();
}

void Dialog::_bookmark_del_callback()
{
  const List<Str> & bookmark = prefs()->bookmark();
  
  if (! bookmark.size()) return;

  dialog::global()->choice(
    dialog_bookmark_del,
    bookmark,
    this,
    bookmark_del_callback);
}

void Dialog::bookmark_del_all_callback(bool)
{
  prefs()->bookmark(List<Str>());

  menu_update();
}

void Dialog::_bookmark_del_all_callback()
{
  if (! prefs()->bookmark().size()) return;
  
  dialog::global()->question(
    dialog_bookmark_del_all,
    this,
    bookmark_del_all_callback);
}

void Dialog::pin_callback(bool) { pin(_p->pin->get()); }

void Dialog::ok_callback(bool)
{
  prefs()->recent_add(dir());

  if (! _pin) hide(); 
  menu_update();

  sig.emit(get());
}

void Dialog::close_callback(bool) { hide(); }

}
}
