/*------------------------------------------------------------------------------
 lib/djv_ui/image.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_ui/image.h>
#include <djv_image/gl_data.h>

namespace djv_ui {
namespace image {

Fl_Image * image_to_fltk(
  const djv_image::Data & in,
  const Vec2i & resize,
  const djv_image::Profile & profile
) {
  using namespace djv_image;
  
  //DJV_DBG_OBJ("fl_image");
  //DJV_DBG_PRN("in = " << in);
  
  // Temporary buffer.
  
  Data tmp(data::Info(vec::size_valid(resize) ? resize : in.size(),
    pixel::RGB_U8));
  //DJV_DBG_PRN("tmp = " << tmp);

  // Transform.
  
  gl_data::Option option;
  option.xform.scale =
    Vec2f(tmp.size()) /
    (Vec2f(in.size() * data::proxy_scale(in.info().proxy)));
  option.xform.mirror.y = true;
  option.profile = profile;
  
  gl_data::copy(in, &tmp, option);

  // Convert to FLTK.
  
  return Fl_RGB_Image(tmp.data(), tmp.w(), tmp.h()).copy();
}

}
}

