/*------------------------------------------------------------------------------
 lib/djv_ui/messages_dialog.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_ui/button.h>
#include <djv_ui/group.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/label.h>
#include <djv_ui/messages_dialog.h>
#include <djv_ui/prefs.h>
#include <djv_ui/text.h>
#include <FL/Fl_Pixmap.H>

namespace djv_ui {
namespace messages_dialog {

struct Dialog::_Dialog
{
  text::Multiline_Display * widget;
  button::Check * show;
  button::Push * clear, * close;
};

Dialog::Dialog() :
  dialog::Base(label_title),
  _show(true),
  _p(new _Dialog)
{
  // Widgets.
  
  _p->widget = new text::Multiline_Display;
  _p->show = new button::Check(dialog::label_show);
  _p->clear = new button::Push(label_clear);
  _p->close = new button::Push(dialog::label_close);
  
  // Layout.
  
  Layout_V * layout = new Layout_V(this);
  layout->add(_p->widget);
  layout->stretch(_p->widget);

  Layout_H * layout_h = new Layout_H(layout);
  layout_h->margin(0);
  layout_h->add(_p->show);
  layout_h->add_stretch();
  layout_h->add(_p->clear);
  layout_h->add(_p->close);
  
  // Preferences.
  
  prefs::Prefs prefs(prefs::prefs_global(), "messages_dialog");
  prefs::get(&prefs, "show", &_show);

  // Initialize.
  
  widget_update();
  dialog_update();
  
  size(size_hint());

  // Callbacks.

  _p->show->sig.set(this, show_callback);
  _p->clear->sig.set(this, clear_callback);
  _p->close->sig.set(this, close_callback);
}

Dialog::~Dialog()
{}

void Dialog::del()
{
  prefs::Prefs prefs(prefs::prefs_global(), "messages_dialog");
  prefs::set(&prefs, "show", _show);

  dialog::Base::del();
}

void Dialog::set(const Str & in)
{
  _list.push_front(in);
  if (_list.size() > 100) _list.pop_back();
  
  widget_update();
  if (_show) show();
}

void Dialog::set(const Error & in, const Str & message)
{
  const Str tmp = djv_base::error::format(in);
  set(message.size() ?
    Str(str::Format(error_format).arg(tmp).arg(message)) : tmp);
}

void Dialog::show()
{
  _p->close->take_focus();

  dialog::Base::show();
}

void Dialog::dirty()
{
  Base::dirty();
  
  _size_min = max(Base::size_min(), Vec2i(400, 200));
}

const Vec2i & Dialog::size_min() const { return _size_min; }

void Dialog::show_callback(bool in)
{
  _show = in;
  
  dialog_update();
}

void Dialog::clear_callback(bool)
{
  _list.clear();
  
  widget_update();
}

void Dialog::close_callback(bool)
{
  hide();
}

void Dialog::widget_update()
{
  callbacks(false);
  
  _p->widget->set(str::join(_list, "\n"));
  
  callbacks(true);
}

void Dialog::dialog_update()
{
  callbacks(false);
  
  _p->show->set(_show);
  
  callbacks(true);
}

const Str
  label_title = "Messages",
  label_clear = "&Clear";

const Str
  error_format = "%%: %%";

void message(const Str & in)
{
  global()->set(in);
}

void error(const Error & in, const Str & message)
{
  global()->set(in, message);
}

}
}

