/*------------------------------------------------------------------------------
 lib/djv_ui/pixel.cxx
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_ui/choice.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/pixel.h>

namespace djv_ui {
namespace pixel {

struct Widget::_Widget
{
  choice::Widget * widget;
};

Widget::Widget() :
  sig(this),
  _value(djv_image::PIXEL(0)),
  _p(new _Widget)
{
  add(_p->widget = new choice::Widget(djv_image::pixel::label_pixel()));
  
  size(layout::FIXED, layout::FIXED);
  align(layout::LEFT, layout::CENTER);
  
  widget_update();
  
  _p->widget->sig.set(this, widget_callback);
}

Widget::~Widget()
{}

void Widget::del()
{
  widget::Widget::del();
  
  _p->widget->del();
}

void Widget::set(djv_image::PIXEL in)
{
  if (in == _value) return;
  
  _value = in;
  
  widget_update();
  sig.emit(_value);
}

djv_image::PIXEL Widget::get() const { return _value; }

void Widget::draw()
{
  widget::Widget::draw();
}

void Widget::dirty()
{
  widget::Widget::dirty();
  
  _p->widget->dirty();
  
  size_hint(_p->widget->size_hint());
}

void Widget::geom(const Box2i & in)
{
  widget::Widget::geom(in);
  
  _p->widget->geom(in);
}

const Box2i & Widget::geom() const { return widget::Widget::geom(); }

void Widget::widget_callback(int in)
{
  set(djv_image::PIXEL(in));
}

void Widget::widget_update()
{
  callbacks(false);
  
  _p->widget->set(_value);
  
  callbacks(true);
}

}
}

