/*------------------------------------------------------------------------------
 lib/djv_ui/vec.cxx
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_ui/button.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/num_int.h>
#include <djv_ui/num_float.h>
#include <djv_ui/vec.h>

namespace djv_ui {
namespace vec {

struct Vec2i::_Vec2i
{
  num::Int_Edit * widget, * widget2;
};

Vec2i::Vec2i() :
  sig(this),
  _p(new _Vec2i)
{
  _p->widget = new num::Int_Edit;
  _p->widget2 = new num::Int_Edit;
  
  size(layout::FIXED, layout::FIXED);
  align(layout::LEFT, layout::CENTER);
  
  Layout_H * layout = new Layout_H(this);
  layout->margin(0);
  layout->add(_p->widget);
  layout->add(_p->widget2);

  widget_update();
  
  _p->widget->sig.set(this, widget_callback);
  _p->widget2->sig.set(this, widget2_callback);
}

Vec2i::~Vec2i() {}

void Vec2i::set(const djv_base::Vec2i & in)
{
  if (in == _value) return;
  
  _value = in;
  
  widget_update();
  sig.emit(_value);
}
  
const djv_base::Vec2i & Vec2i::get() const { return _value; }

void Vec2i::widget_callback(int in) { set(djv_base::Vec2i(in, _value.y)); }
void Vec2i::widget2_callback(int in) { set(djv_base::Vec2i(_value.x, in)); }

void Vec2i::widget_update()
{
  callbacks(false);
  
  _p->widget->set(_value.x);
  _p->widget2->set(_value.y);
  
  callbacks(true);
}

struct Vec2f::_Vec2f
{
  num::Float_Edit * widget, * widget2;
};

Vec2f::Vec2f() :
  sig(this),
  _p(new _Vec2f)
{
  _p->widget = new num::Float_Edit;
  _p->widget2 = new num::Float_Edit;
  
  size(layout::FIXED, layout::FIXED);
  align(layout::LEFT, layout::CENTER);
  
  Layout_H * layout = new Layout_H(this);
  layout->margin(0);
  layout->add(_p->widget);
  layout->add(_p->widget2);

  widget_update();
  
  _p->widget->sig.set(this, widget_callback);
  _p->widget2->sig.set(this, widget2_callback);
}

Vec2f::~Vec2f()
{}

void Vec2f::set(const djv_base::Vec2f & in)
{
  if (in == _value) return;
  
  _value = in;
  
  widget_update();
  sig.emit(_value);
}

const djv_base::Vec2f & Vec2f::get() const { return _value; }

void Vec2f::widget_callback(double in) { set(djv_base::Vec2f(in, _value.y)); }
void Vec2f::widget2_callback(double in) { set(djv_base::Vec2f(_value.x, in)); }

void Vec2f::widget_update()
{
  callbacks(false);
  
  _p->widget->set(_value.x);
  _p->widget2->set(_value.y);
  
  callbacks(true);
}

}
}

