/*------------------------------------------------------------------------------
 plugin/djv_exr_base/base.h

 Image I/O - EXR
 ---------------
 
 Industrial Light & Magic OpenEXR image file format.
 
 Requires:
 
 * OpenEXR - http:://www.openexr.com

 Support:
 
 * Image data: L, LA, RGB, RGBA, 16-bit float, 32-bit float.
 * Image layers.
 * File compression.

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_EXR_BASE_H
#define DJV_EXR_BASE_H

#include <djv_image/pixel.h>
#include <djv_base/box.h>
#include <ImathBox.h>
#include <ImfPixelType.h>

namespace Imf {
class InputFile;
class OutputFile;
}

namespace djv_exr {
using namespace djv_base;
using namespace djv_image;

//------------------------------------------------------------------------------
// Information

static const Str name = "EXR";

static const List<Str> extension_list = List<Str>() << ".exr";

//------------------------------------------------------------------------------
// Image Tags

enum TAG
{
  TAG_LONGITUDE,
  TAG_LATITUDE,
  TAG_ALTITUDE,
  TAG_FOCUS,
  TAG_EXPOSURE,
  TAG_APERTURE,
  TAG_ISO_SPEED,
  TAG_CHROMATICITIES,
  TAG_WHITE_LUMINANCE,
  TAG_X_DENSITY
};

//------------------------------------------------------------------------------
// EXR Data

struct Channel
{
  Channel() :
    sampling(1)
  {}

  Channel(
    const Str & name,
    pixel::TYPE type,
    const Vec2i & sampling = Vec2i(1)
  ) :
    name(name),
    type(type),
    sampling(sampling)
  {}

  Str name;
  pixel::TYPE type;
  Vec2i sampling;
};

//------------------------------------------------------------------------------
// Options

enum COLOR
{
  COLOR_RAW,
  COLOR_EXPOSURE
};

enum COMPRESS
{
  COMPRESS_NONE,
  COMPRESS_RLE,
  COMPRESS_ZIPS,
  COMPRESS_ZIP,
  COMPRESS_PIZ,
  COMPRESS_PXR24,
  COMPRESS_B44,
  COMPRESS_B44A
};

//------------------------------------------------------------------------------
// Conversion

Box2i imf_box(const Imath::Box2i &);

Imf::PixelType imf_type(pixel::TYPE);
pixel::TYPE imf_type(Imf::PixelType);

Str & operator >> (Str &, COLOR &) throw (Str);
Str & operator >> (Str &, COMPRESS &) throw (Str);
Str & operator << (Str &, COLOR);
Str & operator << (Str &, COMPRESS);

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_color();
const List<Str> & label_compress();
const List<Str> & label_tag();

}

#endif

