/*------------------------------------------------------------------------------
 plugin/djv_png/io.h

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_PNG_IO_H
#define DJV_PNG_IO_H

#include "base.h"
#include <djv_image/io.h>

namespace djv_png {
namespace io {
using namespace djv_base;
using namespace djv_image;

//------------------------------------------------------------------------------
// Load Plugin

class Load : public djv_image::io::Load
{
public:

  Load();
  ~Load();

  plugin::Plugin * copy() const;
  const Str & name() const;
  const List<Str> & extension_list() const;
  
  void open(const File &, Info *, Tag *, djv_image::Time *) throw (Error);
  void open(const File &, List<Info> *, Tag *, djv_image::Time *) throw (Error);
  const Image * get(int frame, int layer, data::PROXY) throw (Error);

private:

  void _open(const File &, Info *) throw (Error);
  void _close();

  File _file;
  FILE * _f;
  png_structp _png;
  png_infop _png_info, _png_info_end;
  Png_Error _png_error;
  Image _image;
  Data _tmp;
};

//------------------------------------------------------------------------------
// Save Plugin

class Save : public djv_image::io::Save
{
public:

  Save();
  ~Save();
  
  plugin::Plugin * copy() const;
  const Str & name() const;
  const List<Str> & extension_list() const;
  
  void open(const File &, const Info &, const Tag &, const djv_image::Time &)
    throw (Error);
  void set(const Image &, int frame) throw (Error);

private:

  void _open(const File &, const Tag &) throw (Error);
  void _close();
  
  File _file;
  FILE * _f;
  png_structp _png;
  png_infop _png_info;
  Png_Error _png_error;
  Info _info;
  Image _image;
};

}
}

#endif

