#define DJV_DBG

#include <djv_base/file.h>

using namespace djv_base;

void seq_to_str()
{
  DJV_DBG_OBJ("seq_to_str");
  
  const struct Data { file::Seq a; Str b; } data [] =
  {
    { file::Seq(List<int>()), "" },
    { file::Seq(List<int>() << 1), "1" },
    { file::Seq(List<int>() << 1 << 2), "1-2" },
    { file::Seq(List<int>() << 1 << 2 << 3), "1-3" },
    { file::Seq(List<int>() << 1 << 2 << 3, 4), "0001-0003" },
    { file::Seq(List<int>() << 3 << 2 << 1), "3-1" },
    { file::Seq(List<int>() << 1 << 2 << 3 << 5), "1-3,5" },
    { file::Seq(List<int>() << 1 << 2 << 3 << 5 << 6), "1-3,5-6" },
    { file::Seq(List<int>() << 1 << 2 << 3 << 5 << 6 << 7), "1-3,5-7" },
    { file::Seq(List<int>() << 1 << 2 << 3 << 3 << 2 << 1), "1-3,3-1" }
  };
  const int data_size = sizeof(data) / sizeof(Data);

  for (int i = 0; i < data_size; ++i)
  {
    const Str tmp = file::seq_to_str(data[i].a);
    DJV_DBG_PRN(data[i].a.list << " = " << tmp);
    DJV_ASSERT(tmp == data[i].b);
  }
}

void str_to_seq()
{
  DJV_DBG_OBJ("str_to_seq");
  
  const struct Data { Str a; file::Seq b; } data [] =
  {
    { "", file::Seq(List<int>() << 0) },
    { "1", file::Seq(List<int>() << 1) },
    { "1-2", file::Seq(List<int>() << 1 << 2) },
    { "1-3", file::Seq(List<int>() << 1 << 2 << 3) },
    { "0001-0003", file::Seq(List<int>() << 1 << 2 << 3, 4) },
    { "3-1", file::Seq(List<int>() << 3 << 2 << 1) },
    { "1,2,3", file::Seq(List<int>() << 1 << 2 << 3) },
    { "1-3,5", file::Seq(List<int>() << 1 << 2 << 3 << 5) },
    { "1-3,5-6", file::Seq(List<int>() << 1 << 2 << 3 << 5 << 6) },
    { "1-3,5-7", file::Seq(List<int>() << 1 << 2 << 3 << 5 << 6 << 7) },
    { "1-3,3-1", file::Seq(List<int>() << 1 << 2 << 3 << 3 << 2 << 1) }
  };
  const int data_size = sizeof(data) / sizeof(Data);

  for (int i = 0; i < data_size; ++i)
  {
    const file::Seq tmp = file::str_to_seq(data[i].a);
    DJV_DBG_PRN(data[i].a << " = " << tmp);
    DJV_ASSERT(tmp == data[i].b);
  }
}

int main(int argc, char ** argv)
{
  seq_to_str();
  str_to_seq();
  
  return 0;
}

