//#define DJV_DBG

#include <djv_ui/layout_row.h>
#include <djv_ui/choice.h>
#include <djv_ui/pixel.h>
#include <djv_ui/image_view.h>
#include <djv_ui/window.h>
#include <djv_ui/app.h>

class App : public djv_ui::app::App
{
public:

  App(int & argc, char ** argv);
  ~App();

private:

  DJV_CALLBACK(App, pixel_callback, djv_image::PIXEL);
  DJV_CALLBACK(App, exit_callback, bool) { exit(); }
  
  void update();

  djv_image::Image _image, _gradient;
  djv_image::PIXEL _pixel;
  
  djv_ui::image_view::Widget * _widget;
  djv_ui::pixel::Widget * _pixel_widget;
  djv_ui::window::Widget * _window;
};

App::App(int & argc, char ** argv) :
  djv_ui::app::App("image_view", argc, argv),
  _pixel(djv_image::PIXEL(0)),
  _window(0)
{
  using namespace djv_base;
  using namespace djv_ui;

  valid(true);

  // Widgets.

  _widget = new image_view::Widget;
  _pixel_widget = new pixel::Widget;
  _window = new window::Widget(name());
  
  // Layout.

  Layout_V * layout = new Layout_V(_window);
  layout->add(_widget);
  layout->stretch(_widget);
  layout->add(_pixel_widget);
  
  // Initialize.
  
  _gradient.set(djv_image::Info(256, djv_image::pixel::L_F32));
  djv_image::data::gradient(&_gradient);

  //update();
  
  _widget->set(&_image);
  
  _window->size(256);
  _window->show();

  // Callbacks.

  _pixel_widget->sig.set(this, pixel_callback);
  _window->close_sig.set(this, exit_callback);
}

App::~App()
{
  if (_window) _window->del();
}

void App::pixel_callback(djv_image::PIXEL in)
{
  _pixel = in;
  
  update();
}
  
void App::update()
{
  DJV_DBG_OBJ("App::update");
  
  _image.set(djv_image::Info(_gradient.size(), _pixel));
  djv_image::gl_data::copy(_gradient, &_image);
  
  _widget->redraw();
  _pixel_widget->set(_pixel);
}

int main(int argc, char ** argv) { return App(argc, argv).exec(); }

